import * as minecraft_main_server from "@minecraft/server";
import { ActionFormData, ActionFormResponse } from '@minecraft/server-ui';
import * as Quest from './libraries/quest/QuestClass';
import * as questConstants from './libraries/quest/questConstants';
import * as lib0 from './libraries/getRandomInt';

//General quest functions

export function RecoverGatherQuest(player) {
	let activeGatherQuest = new Quest.Gather(player);
	activeGatherQuest.gatherItem = JSON.parse(player.getDynamicProperty("better:activeGatherQuest")).gatherItem;
	activeGatherQuest.gatherAmount = JSON.parse(player.getDynamicProperty("better:activeGatherQuest")).gatherAmount;
	activeGatherQuest.questText = JSON.parse(player.getDynamicProperty("better:activeGatherQuest")).questText;
	activeGatherQuest.returnItemType = JSON.parse(player.getDynamicProperty("better:activeGatherQuest")).returnItemType;
	activeGatherQuest.typeId = JSON.parse(player.getDynamicProperty("better:activeGatherQuest")).typeId;

	activeGatherQuest.openUi();
}
export function RecoverHuntQuest(player) {
	let activeHuntQuest = new Quest.Hunt(player);
	activeHuntQuest.killedEntities = player.getDynamicProperty("better:hunt_quest_killed_entities");
	activeHuntQuest.killEntityTypeId = JSON.parse(player.getDynamicProperty("better:activeHuntQuest")).killEntityTypeId;
	activeHuntQuest.killEntityAmount = JSON.parse(player.getDynamicProperty("better:activeHuntQuest")).killEntityAmount;
	activeHuntQuest.questText = JSON.parse(player.getDynamicProperty("better:activeHuntQuest")).questText;
	activeHuntQuest.returnItemType = JSON.parse(player.getDynamicProperty("better:activeHuntQuest")).returnItemType;
	activeHuntQuest.typeId = JSON.parse(player.getDynamicProperty("better:activeHuntQuest")).typeId;

	activeHuntQuest.openUi();
}

export function IncrementPlayerEntityKillCount(entity, player) {
	if (player.getDynamicProperty("better:activeHuntQuest")) {
		if (entity.typeId == JSON.parse(player.getDynamicProperty("better:activeHuntQuest")).killEntityTypeId[0]) {
			let n = player.getDynamicProperty("better:hunt_quest_killed_entities") + 1;
			player.setDynamicProperty("better:hunt_quest_killed_entities", n);
		}
	}
}

//Quest types

export function townGatherQuest(player) {
	if (player.getDynamicProperty("better:is_questing_gather") == true && JSON.parse(player.getDynamicProperty("better:activeGatherQuest")).typeId == "townQuest") {
		RecoverGatherQuest(player);
	} else if (player.getDynamicProperty("better:is_questing_gather") == true && JSON.parse(player.getDynamicProperty("better:activeGatherQuest")).typeId != "townQuest") {
			player.runCommandAsync('title @s actionbar You have to complete your unfinished quests first.');
	} else {
		let activeGatherQuest = new Quest.Gather(player);
		const gatherItems = questConstants.townGatherQuestItems;
		activeGatherQuest.gatherItem = gatherItems[lib0.getRandomInt(0, (gatherItems.length - 1))];
		activeGatherQuest.gatherAmount = lib0.getRandomInt(2, 6);
		const texts = [
			`Dear Residents,\n\nThat Illager raid sure has been tragic, we are indeed severely lacking resources. It would be a great help for the village to help un in the collection of§p ${activeGatherQuest.gatherItem[1]}§r. \n\nWho thinks thez are able, let them help us.\n\nThank you for your cooperation.`,
			`Hey folks,\n\nWe're trying to spruce up the town square a bit, and we could use some help gathering§p ${activeGatherQuest.gatherAmount} ${activeGatherQuest.gatherItem[1]}§r. If you've got some time to spare, come lend a hand!\n\nThanks a ton!`,
			`Hi everyone,\n\nThe festival's coming up, and we're short on decorations. If you're willing to help gather§p ${activeGatherQuest.gatherAmount} ${activeGatherQuest.gatherItem[1]}§r, swing by and pitch in!\n\nMuch appreciated!`,
			`Hey neighbors,\n\nWe've got a mess of debris to clear after the rain. If you can spare some time to help gather ${activeGatherQuest.gatherAmount} ${activeGatherQuest.gatherItem[1]}, we'd be really grateful!\n\nThanks for your help!`,
			`Hey there,\n\nWe need to beef up our defenses against those undead attacks. If you're willing to help gather§p ${activeGatherQuest.gatherAmount} ${activeGatherQuest.gatherItem[1]}§r, it'd be a huge help!\n\nThanks a bunch!`,
			`Hey folks,\n\nWe're running low on §p${activeGatherQuest.gatherItem[1]}§r, and we could really use some extra hands for mining. If you're up for it, grab §p${activeGatherQuest.gatherAmount}§r of them and bring them over. We'd really appreciate it!\n\nThanks a bunch!`
		];
		activeGatherQuest.questText = texts[lib0.getRandomInt(0, (texts.length - 1))];
		activeGatherQuest.returnItemType = "better:town_gather_quest_complete";
		activeGatherQuest.typeId = "townQuest";

		activeGatherQuest.openUi();
		player.setDynamicProperty("better:activeGatherQuest", JSON.stringify(activeGatherQuest));
	}
}

export function townHuntQuest(player) {
	if (player.getDynamicProperty("better:is_questing_hunt") == true && JSON.parse(player.getDynamicProperty("better:activeHuntQuest")).typeId == "townQuest") {
		RecoverHuntQuest(player);
	} else if (player.getDynamicProperty("better:is_questing_hunt") == true && JSON.parse(player.getDynamicProperty("better:activeHuntQuest")).typeId != "townQuest") {
			player.runCommandAsync('title @s actionbar Finish up your other quests first!');
	} else {
		player.setDynamicProperty("better:hunt_quest_killed_entities", 0);
		let activeHuntQuest = new Quest.Hunt(player);
		const killEntities = questConstants.townHuntQuestEntities;
		activeHuntQuest.killEntityTypeId = killEntities[lib0.getRandomInt(0, (killEntities.length - 1))];
		activeHuntQuest.killEntityAmount = lib0.getRandomInt(2, 6);
		const texts = [
			`Hey folks,\n\nWe've been having some trouble with §p${activeHuntQuest.killEntityTypeId[1]}§r around here, and we could really use your help. If you're up for it, take out §p${activeHuntQuest.killEntityAmount}§r of them and let us know!\n\nThanks a bunch!`,
			`Hey neighbors,\n\nThose pesky §p${activeHuntQuest.killEntityTypeId[1]}§r have been causing a lot of trouble lately. If you've got the guts to take out §p${activeHuntQuest.killEntityAmount}§r of them, swing by and lend a hand!\n\nMuch appreciated!`,
			`Hi everyone,\n\nWe've got a bit of a monster problem on our hands. If you're up for it, take out §p${activeHuntQuest.killEntityAmount}§r of those §p${activeHuntQuest.killEntityTypeId[1]}§r and bring us the proof!\n\nThanks for your help!`,
			`Hey there,\n\nWe've been dealing with some serious beasties lately. If you're willing to help take out §p${activeHuntQuest.killEntityAmount}§r of those §p${activeHuntQuest.killEntityTypeId[1]}§r, it'd be a huge help!\n\nThanks a bunch!`,
			`Hey everyone,\n\nWe've got a bit of a monster problem on our hands. If you're up for it, take out §p${activeHuntQuest.killEntityAmount}§r of those §p${activeHuntQuest.killEntityTypeId[1]}§r and bring us the proof!\n\nThanks for your help!`,
			`Hey folks,\n\nThose §p${activeHuntQuest.killEntityTypeId[1]}§r have been causing a real ruckus lately. If you're up for it, take out §p${activeHuntQuest.killEntityAmount}§r of them and let us know!\n\nThanks a bunch!`
		];
		activeHuntQuest.questText = texts[lib0.getRandomInt(0, (texts.length - 1))];
		activeHuntQuest.returnItemType = "better:town_hunt_quest_complete";
		activeHuntQuest.typeId = "townQuest";

		activeHuntQuest.openUi();
		player.setDynamicProperty("better:activeHuntQuest", JSON.stringify(activeHuntQuest));
	}
}